import { Token } from "../types/token.ts";
import { ASTNode } from "../types/token.ts";

export class Parser {
    private position = 0;

    constructor(private tokens: Token[]) {}

    private match(type: string, value?: string): boolean {
        const token = this.tokens[this.position];
        if (!token) return false;
        if (value !== undefined) {
            return token.type === type && token.value === value;
        }
        return token.type === type;
    }

    private peek(): Token | null {
        return this.tokens[this.position] || null;
    }

    parseExpressions(): ASTNode[] {
        const expressions: ASTNode[] = [];
        while (this.position < this.tokens.length) {
            const expr = this.parseExpression();
            if (expr) {
                expressions.push(expr);
            }
        }
        return expressions;
    }

    private parseExpression(): ASTNode | null {
        const token = this.peek();
        if (!token) return null;

        if (token.type === "IDENTIFIER") {
            this.position++;
            if (this.match("SYMBOL", "[")) {
                return this.parseIndexingOrFunctionCall(token);
            }
            return { type: "Identifier", value: token.value };
        }

        if (token.type === "NUMBER") {
            this.position++;
            return { type: "Number", value: token.value };
        }

        if (token.type === "OPERATOR") {
            return this.parseOperatorOrUnary();
        }

        throw new Error(`Unexpected token: ${token.value}`);
    }

    private parseIndexingOrFunctionCall(identifier: Token): ASTNode {
        const args: ASTNode[] = [];
        this.position++; // Consume "["

        while (!this.match("SYMBOL", "]") && !this.match("SYMBOL", "}")) {
            const expr = this.parseExpression();
            if (!expr) {
                throw new Error("Expected expression inside indexing or function call");
            }
            args.push(expr);
            if (this.match("SYMBOL", ";")) {
                this.position++;
            }
        }

        const closingSymbol = this.tokens[this.position].value;
        this.position++; // Consume closing bracket or brace

        return {
            type: closingSymbol === "]" ? "FunctionCall" : "Indexing",
            name: identifier.value,
            args
        };
    }

    private parseOperatorOrUnary(): ASTNode {
        const operator = this.tokens[this.position];
        this.position++;

        if (this.isUnaryOperator(operator.value)) {
            const operand = this.parseExpression();
            return {
                type: "UnaryExpression",
                operator: operator.value,
                operand
            };
        }

        const left = this.parseExpression();
        const right = this.parseExpression();

        if (this.isNonCommutative(operator.value)) {
            return {
                type: "BinaryExpression",
                operator: operator.value,
                left: right,
                right: left
            };
        }

        return {
            type: "BinaryExpression",
            operator: operator.value,
            left,
            right
        };
    }

    private isUnaryOperator(op: string): boolean {
        return ["!", "#", "_", "$"].includes(op);
    }

    private isNonCommutative(op: string): boolean {
        return ["-", "%", "!", "^"].includes(op);
    }
} 